@@ -29,8 +29,6 @@
 #include <glib/gi18n.h>
 #include <gtk/gtk.h>
 
-#include <unique/uniqueapp.h>
-
 #include <bluetooth-client.h>
 #include <bluetooth-client-private.h>
 #include <bluetooth-chooser.h>
@@ -40,6 +38,7 @@
 #include "agent.h"
 
 static gboolean option_debug = FALSE;
+static gboolean option_dump = FALSE;
 static BluetoothClient *client;
 static GtkTreeModel *devices_model;
 static guint num_adapters_present = 0;
@@ -50,10 +49,10 @@ static gboolean discover_lock = FALSE;
 #define SCHEMA_NAME		"org.gnome.Bluetooth"
 #define PREF_SHOW_ICON		"show-icon"
 
-#define KEYBOARD_PREFS		"gnome-keyboard-properties"
-#define MOUSE_PREFS		"gnome-mouse-properties"
-#define SOUND_PREFS		"gnome-volume-control"
-#define SOUND_PREFS_CMDLINE	SOUND_PREFS " -p hardware"
+#define GNOMECC			"gnome-control-center"
+#define KEYBOARD_PREFS		GNOMECC " keyboard"
+#define MOUSE_PREFS		GNOMECC " mouse"
+#define SOUND_PREFS		GNOMECC " sound"
 
 enum {
 	CONNECTED,
@@ -78,7 +77,6 @@ void wizard_callback(GObject *widget, gpointer user_data);
 void sendto_callback(GObject *widget, gpointer user_data);
 
 static void action_set_bold (GtkUIManager *manager, GtkAction *action, const char *path);
-static void update_icon_visibility (void);
 
 void quit_callback(GObject *widget, gpointer user_data)
 {
@@ -248,7 +246,7 @@ static void mouse_callback(GObject *widget, gpointer user_data)
 
 static void sound_callback(GObject *widget, gpointer user_data)
 {
-	const char *command = SOUND_PREFS_CMDLINE;
+	const char *command = SOUND_PREFS;
 
 	if (!g_spawn_command_line_async(command, NULL))
 		g_printerr("Couldn't execute command: %s\n", command);
@@ -262,45 +260,6 @@ void wizard_callback(GObject *widget, gpointer user_data)
 		g_printerr("Couldn't execute command: %s\n", command);
 }
 
-static gboolean
-set_powered_foreach (GtkTreeModel *model,
-		     GtkTreePath  *path,
-		     GtkTreeIter  *iter,
-		     gpointer      data)
-{
-	DBusGProxy *proxy = NULL;
-	GValue value = { 0, };
-
-	gtk_tree_model_get (model, iter,
-			    BLUETOOTH_COLUMN_PROXY, &proxy, -1);
-	if (proxy == NULL)
-		return FALSE;
-
-	g_value_init (&value, G_TYPE_BOOLEAN);
-	g_value_set_boolean (&value, TRUE);
-
-	dbus_g_proxy_call_no_reply (proxy, "SetProperty",
-				    G_TYPE_STRING, "Powered",
-				    G_TYPE_VALUE, &value,
-				    G_TYPE_INVALID,
-				    G_TYPE_INVALID);
-
-	g_value_unset (&value);
-	g_object_unref (proxy);
-
-	return FALSE;
-}
-
-static void
-bluetooth_set_adapter_powered (void)
-{
-	GtkTreeModel *adapters;
-
-	adapters = bluetooth_client_get_adapter_model (client);
-	gtk_tree_model_foreach (adapters, set_powered_foreach, NULL);
-	g_object_unref (adapters);
-}
-
 void bluetooth_status_callback (GObject *widget, gpointer user_data)
 {
 	GObject *object;
@@ -312,7 +271,6 @@ void bluetooth_status_callback (GObject *widget, gpointer user_data)
 	bluetooth_killswitch_set_state (killswitch,
 					active ? KILLSWITCH_STATE_UNBLOCKED : KILLSWITCH_STATE_SOFT_BLOCKED);
 	g_object_set_data (object, "bt-active", GINT_TO_POINTER (active));
-	bluetooth_set_adapter_powered ();
 }
 
 void
@@ -322,7 +280,7 @@ bluetooth_discoverable_callback (GtkToggleAction *toggleaction, gpointer user_da
 
 	discoverable = gtk_toggle_action_get_active (toggleaction);
 	discover_lock = TRUE;
-	bluetooth_client_set_discoverable (client, discoverable);
+	bluetooth_client_set_discoverable (client, discoverable, 0);
 	discover_lock = FALSE;
 }
 
@@ -372,7 +330,6 @@ killswitch_state_changed (BluetoothKillswitch *killswitch, KillswitchState state
 	if (state == KILLSWITCH_STATE_NO_ADAPTER) {
 		object = gtk_builder_get_object (xml, "bluetooth-applet-popup");
 		gtk_menu_popdown (GTK_MENU (object));
-		update_icon_visibility ();
 		return;
 	}
 
@@ -407,8 +364,6 @@ killswitch_state_changed (BluetoothKillswitch *killswitch, KillswitchState state
 
 	object = gtk_builder_get_object (xml, "bluetooth-applet-ui-manager");
 	gtk_ui_manager_ensure_update (GTK_UI_MANAGER (object));
-
-	update_icon_visibility ();
 }
 
 static GtkWidget *create_popupmenu(void)
@@ -514,7 +469,6 @@ update_discoverability (GtkTreeIter *iter)
 	object = gtk_builder_get_object (xml, "discoverable");
 
 	if (iter == NULL) {
-		discover_lock = FALSE;
 		gtk_action_set_visible (GTK_ACTION (object), FALSE);
 		return;
 	}
@@ -937,7 +891,7 @@ update_device_list (GtkTreeIter *parent)
 
 			add_separator_item (address, "files-sep", uimanager, menu_merge_id);
 
-			if (type == BLUETOOTH_TYPE_KEYBOARD && program_available (KEYBOARD_PREFS)) {
+			if (type == BLUETOOTH_TYPE_KEYBOARD && program_available (GNOMECC)) {
 				add_menu_item (address,
 					       "keyboard",
 					       _("Open Keyboard Preferences..."),
@@ -945,7 +899,7 @@ update_device_list (GtkTreeIter *parent)
 					       menu_merge_id,
 					       G_CALLBACK (keyboard_callback));
 			}
-			if (type == BLUETOOTH_TYPE_MOUSE && program_available (MOUSE_PREFS)) {
+			if (type == BLUETOOTH_TYPE_MOUSE && program_available (GNOMECC)) {
 				add_menu_item (address,
 					       "mouse",
 					       _("Open Mouse Preferences..."),
@@ -955,7 +909,7 @@ update_device_list (GtkTreeIter *parent)
 			}
 			if ((type == BLUETOOTH_TYPE_HEADSET ||
 			     type == BLUETOOTH_TYPE_HEADPHONES ||
-			     type == BLUETOOTH_TYPE_OTHER_AUDIO) && program_available (SOUND_PREFS)) {
+			     type == BLUETOOTH_TYPE_OTHER_AUDIO) && program_available (GNOMECC)) {
 				add_menu_item (address,
 					       "sound",
 					       _("Open Sound Preferences..."),
@@ -1077,14 +1031,35 @@ show_icon_changed (GSettings *settings,
 	update_icon_visibility();
 }
 
+static void
+dump_devices (void)
+{
+	BluetoothClient *client;
+	GtkTreeModel *model;
+	GtkTreeIter iter;
+
+	client = bluetooth_client_new ();
+	model = bluetooth_client_get_model (client);
+
+	if (gtk_tree_model_get_iter_first (model, &iter) == FALSE) {
+		g_print ("No known devices\n");
+		g_print ("Is bluetoothd running, and a Bluetooth adapter enabled?\n");
+		return;
+	}
+	bluetooth_client_dump_device (model, &iter, TRUE);
+	while (gtk_tree_model_iter_next (model, &iter))
+		bluetooth_client_dump_device (model, &iter, TRUE);
+}
+
 static GOptionEntry options[] = {
 	{ "debug", 'd', 0, G_OPTION_ARG_NONE, &option_debug, N_("Debug"), NULL },
+	{ "dump-devices", 'u', 0, G_OPTION_ARG_NONE, &option_dump, N_("Output a list of currently known devices"), NULL },
 	{ NULL },
 };
 
 int main(int argc, char *argv[])
 {
-	UniqueApp *app;
+	GtkApplication *app;
 	GtkStatusIcon *statusicon;
 	GtkWidget *menu;
 	GOptionContext *context;
@@ -1109,8 +1084,9 @@ int main(int argc, char *argv[])
 	}
 
 	if (option_debug == FALSE) {
-		app = unique_app_new ("org.gnome.Bluetooth.applet", NULL);
-		if (unique_app_is_running (app)) {
+		app = gtk_application_new ("org.gnome.Bluetooth.applet",
+					   &argc, &argv);
+		if (g_application_is_remote (G_APPLICATION (app))) {
 			gdk_notify_startup_complete ();
 			g_warning ("Applet is already running, exiting");
 			return 0;
@@ -1119,6 +1095,11 @@ int main(int argc, char *argv[])
 		app = NULL;
 	}
 
+	if (option_dump != FALSE) {
+		dump_devices ();
+		return 0;
+	}
+
 	g_set_application_name(_("Bluetooth Applet"));
 
 	gtk_window_set_default_icon_name("bluetooth");
@@ -1139,19 +1120,13 @@ int main(int argc, char *argv[])
 			 G_CALLBACK(device_removed), NULL);
 	g_signal_connect (G_OBJECT (devices_model), "row-changed",
 			  G_CALLBACK (device_changed), NULL);
-
-	/* Set the default adapter */
+	/* Set the default */
 	device_changed (devices_model, NULL, NULL, NULL);
 	if (bluetooth_killswitch_has_killswitches (killswitch) != FALSE) {
 		killswitch_state_changed (killswitch,
 					  bluetooth_killswitch_get_state (killswitch));
 	}
 
-	/* Make sure all the unblocked adapters are powered,
-	 * so as to avoid seeing unpowered, but unblocked
-	 * devices */
-	bluetooth_set_adapter_powered ();
-
 	settings = g_settings_new (SCHEMA_NAME);
 	show_icon_pref = g_settings_get_boolean (settings, PREF_SHOW_ICON);
 
