@@ -1579,6 +1579,7 @@ gboolean bluetooth_client_stop_discovery(BluetoothClient *client)
  * bluetooth_client_set_discoverable:
  * @client: a #BluetoothClient object
  * @discoverable: whether the device should be discoverable
+ * @timeout: timeout in seconds for making undiscoverable, or 0 for never.
  *
  * Sets the default adapter's discoverable status.
  *
@@ -1586,11 +1587,13 @@ gboolean bluetooth_client_stop_discovery(BluetoothClient *client)
  **/
 gboolean
 bluetooth_client_set_discoverable (BluetoothClient *client,
-				   gboolean discoverable)
+				   gboolean discoverable,
+                                   guint timeout)
 {
+	GError *error = NULL;
 	DBusGProxy *adapter;
 	GValue disco = { 0 };
-	GValue timeout = { 0 };
+	GValue timeoutv = { 0 };
 	gboolean ret;
 
 	g_return_val_if_fail (BLUETOOTH_IS_CLIENT (client), FALSE);
@@ -1602,29 +1605,37 @@ bluetooth_client_set_discoverable (BluetoothClient *client,
 		return FALSE;
 
 	g_value_init (&disco, G_TYPE_BOOLEAN);
-	g_value_init (&timeout, G_TYPE_UINT);
+	g_value_init (&timeoutv, G_TYPE_UINT);
 
 	g_value_set_boolean (&disco, discoverable);
-	g_value_set_uint (&timeout, 0);
+	g_value_set_uint (&timeoutv, timeout);
 
-	ret = dbus_g_proxy_call (adapter, "SetProperty", NULL,
+	if (discoverable) {
+		ret = dbus_g_proxy_call (adapter, "SetProperty", &error,
+					 G_TYPE_STRING, "DiscoverableTimeout",
+					 G_TYPE_VALUE, &timeoutv,
+					 G_TYPE_INVALID, G_TYPE_INVALID);
+		if (ret == FALSE)
+			goto bail;
+	}
+
+	ret = dbus_g_proxy_call (adapter, "SetProperty", &error,
 				 G_TYPE_STRING, "Discoverable",
 				 G_TYPE_VALUE, &disco,
 				 G_TYPE_INVALID, G_TYPE_INVALID);
-	if (ret == FALSE)
-		goto bail;
 
-	ret = dbus_g_proxy_call (adapter, "SetProperty", NULL,
-				 G_TYPE_STRING, "DiscoverableTimeout",
-				 G_TYPE_VALUE, &timeout,
-				 G_TYPE_INVALID, G_TYPE_INVALID);
 
 bail:
 	g_value_unset (&disco);
-	g_value_unset (&timeout);
+	g_value_unset (&timeoutv);
 
 	g_object_unref(adapter);
 
+	if (error) {
+		g_warning ("Cannot set discoverable: %s", error->message);
+		g_error_free (error);
+	}
+
 	return ret;
 }
 
@@ -1979,17 +1990,8 @@ gboolean bluetooth_client_disconnect_service (BluetoothClient *client,
 static void
 services_foreach (const char *service, gpointer _value, GString *str)
 {
-	GEnumClass *eclass;
-	GEnumValue *ev;
-	BluetoothStatus status = GPOINTER_TO_INT (_value);
-
-	eclass = g_type_class_ref (BLUETOOTH_TYPE_STATUS);
-	ev = g_enum_get_value (eclass, status);
-	if (ev == NULL)
-		g_warning ("Unknown status value %d", status);
-
-	g_string_append_printf (str, "%s (%s) ", service, ev ? ev->value_nick : "unknown");
-	g_type_class_unref (eclass);
+	gboolean value = GPOINTER_TO_INT (_value);
+	g_string_append_printf (str, "%s (%s) ", service, value ? "connected" : "not connected");
 }
 
 void
