@@ -31,9 +31,7 @@
 #include <gdk/gdkkeysyms.h>
 
 #include <dbus/dbus-glib.h>
-#include <unique/uniqueapp.h>
 
-#include <bling-spinner.h>
 #include <bluetooth-client.h>
 #include <bluetooth-client-private.h>
 #include <bluetooth-chooser.h>
@@ -505,7 +503,7 @@ void prepare_callback (GtkWidget *assistant,
 
 		complete = FALSE;
 
-		bling_spinner_start (BLING_SPINNER (spinner_connecting));
+		gtk_spinner_start (GTK_SPINNER (spinner_connecting));
 
 		/* translators:
 		 * The '%s' is the device name, for example:
@@ -515,7 +513,7 @@ void prepare_callback (GtkWidget *assistant,
 		gtk_label_set_text (GTK_LABEL (label_connecting), text);
 		g_free (text);
 	} else {
-		bling_spinner_stop (BLING_SPINNER (spinner_connecting));
+		gtk_spinner_stop (GTK_SPINNER (spinner_connecting));
 	}
 
 	if ((page == page_setup || page == page_connecting) && (create_started == FALSE)) {
@@ -564,7 +562,7 @@ void prepare_callback (GtkWidget *assistant,
 
 		complete = FALSE;
 
-		bling_spinner_start (BLING_SPINNER (spinner_finishing));
+		gtk_spinner_start (GTK_SPINNER (spinner_finishing));
 
 		/* translators:
 		 * The '%s' is the device name, for example:
@@ -574,7 +572,7 @@ void prepare_callback (GtkWidget *assistant,
 		gtk_label_set_text (GTK_LABEL (label_finishing), text);
 		g_free (text);
 	} else {
-		bling_spinner_stop (BLING_SPINNER (spinner_finishing));
+		gtk_spinner_stop (GTK_SPINNER (spinner_finishing));
 	}
 
 	if (page == page_summary) {
@@ -582,8 +580,11 @@ void prepare_callback (GtkWidget *assistant,
 		GValue value = { 0, };
 		char **uuids, *text;
 
+		/* FIXME remove this code when bluetoothd has pair/unpair code */
+		g_object_set (G_OBJECT (selector), "device-selected", target_address, NULL);
+
 		bluetooth_chooser_get_selected_device_info (selector, "name", &value);
-		text = g_strdup_printf (_("Successfully setup new device '%s'"), g_value_get_string (&value));
+		text = g_strdup_printf (_("Successfully set up new device '%s'"), g_value_get_string (&value));
 		g_value_unset (&value);
 		gtk_label_set_text (GTK_LABEL (label_summary), text);
 		g_free (text);
@@ -980,16 +981,12 @@ create_wizard (void)
 	return assistant;
 }
 
-static UniqueResponse
-message_received_cb (UniqueApp         *app,
-                     int                command,
-                     UniqueMessageData *message_data,
-                     guint              time_,
-                     gpointer           user_data)
+static void
+activated_cb (GtkApplication    *app,
+	      GVariant          *variant,
+	      gpointer           user_data)
 {
         gtk_window_present (GTK_WINDOW (user_data));
-
-        return UNIQUE_RESPONSE_OK;
 }
 
 static GOptionEntry options[] = {
@@ -998,7 +995,7 @@ static GOptionEntry options[] = {
 
 int main (int argc, char **argv)
 {
-	UniqueApp *app;
+	GtkApplication *app;
 	GError *error = NULL;
 
 	bindtextdomain(GETTEXT_PACKAGE, LOCALEDIR);
@@ -1016,9 +1013,9 @@ int main (int argc, char **argv)
 		return 1;
 	}
 
-	app = unique_app_new ("org.gnome.Bluetooth.wizard", NULL);
-	if (unique_app_is_running (app)) {
-		unique_app_send_message (app, UNIQUE_ACTIVATE, NULL);
+	app = gtk_application_new ("org.gnome.Bluetooth.wizard", &argc, &argv);
+	if (g_application_is_remote (G_APPLICATION (app))) {
+		gdk_notify_startup_complete ();
 		return 0;
 	}
 
@@ -1042,11 +1039,13 @@ int main (int argc, char **argv)
 	window_assistant = create_wizard();
 	if (window_assistant == NULL)
 		return 1;
+	gtk_application_add_window (app,
+				    GTK_WINDOW (window_assistant));
 
-	g_signal_connect (app, "message-received",
-			  G_CALLBACK (message_received_cb), window_assistant);
+	g_signal_connect (app, "activated",
+			  G_CALLBACK (activated_cb), window_assistant);
 
-	gtk_main();
+	gtk_application_run (app);
 
 	bluetooth_plugin_manager_cleanup ();
 
